<?php

require_once 'class.trip_class.inc.php';

class dbClass extends PDO {
		
		var $schedule_db	= "salestool";
		var $salestool_db	= "salestool";
		var $paxtool_db		= "paxtool_siren";
		var $bms_db			= "bms";

		var $error_message;			//  If an error occurs in data retrieval this will contain the error message.
		
		
		
		/**
		 * Function		: isValidKey()
		 * 
		 * Determine whether the API key is valid. 
		 * 
		 * @param string $api_key
		 * @return boolean
		 */
		function isValidKey( $api_key="" ) {
			
			$sql_query = "
					
					SELECT		COUNT(*)			AS api_matches
					
					FROM		webapi_users
						AS		users
					
					WHERE		users.api_key = :api_key;
					
					";
			
			$sql = $this->prepare( $sql_query );
			$variables = array( ':api_key'=>$api_key );
			$sql->execute( $variables );
			
			$is_valid_key = ( (int)$sql->fetchColumn() == 1 );
			
			return( (( DEBUG ) ? true : $is_valid_key));
		}
		
		
		
		
		/**
		 * Function		: getVesselList()
		 * 
		 * Get a list of operating vessels.
		 * 
		 */
		function getVesselList( ) {
			
			$sql_query = "
					
					SELECT 		vessels.vessel_name		AS vessel_name
					
					FROM		{$this->schedule_db}.ml_vessels
						AS		vessels
						
					WHERE 		vessels.vessel_end_operation >= NOW()
						
					ORDER BY	vessel_name;

					";
			
			$sql = $this->prepare( $sql_query );
			$sql->execute();
			
			$a_vessels = $sql->fetchAll( PDO::FETCH_OBJ );
			
			return( $a_vessels );		
			
		}
		
		
		/**
		 * Function			: getLastTripDate()
		 * s
		 * @param string $vessel
		 * @return mixed
		 */
		function getLastTripDate( $vessel="" ) {
			
			//  If a vessel is defined, match either the name or the key.
			//
			$vessel_filter = ( $vessel == "" ) ? "1" : "( vessels.vessel_key = :vessel ) OR ( vessels.vessel_name = :vessel)";
			
			$sql_query = "
					
					SELECT		trips.trip_start_date		AS trip_date
					
					FROM		{$this->schedule_db}.ml_trips
						AS		trips
						
					JOIN		{$this->schedule_db}.ml_vessels
						AS		vessels
						ON		vessels.vessel_key = trips.trip_vessel
						
					WHERE		{$vessel_filter}
			
					ORDER BY	trip_date DESC
					
					LIMIT 		1;
					
					";
			
			$sql = $this->prepare( $sql_query );
			$sql->execute( array( ':vessel'=>$vessel ));
			
			$last_trip_date = $sql->fetchColumn();
			
			return( $last_trip_date );			
		}
		
		
		
		
		// Get a list of the cabin classes in use on a given vessel between two dates.
		//
		function getActiveCabinClasses( $vessel="", $start_date="", $end_date="" ) {
			
			//  If a vessel is defined, match either the name or the key.
			//
			$vessel_filter = ( $vessel == "" )
				? "1"
				: "( vessels.vessel_key = :vessel ) OR ( vessels.vessel_name = :vessel)";
						
			$sql_query = "
			
					SELECT			classes.class_key			AS class_key,
									classes.class_name			AS class_name,
									classes.class_sort_order	AS class_sort_order
			
					FROM			{$this->schedule_db}.ml_classes
					  AS			classes
			
					JOIN			{$this->schedule_db}.ml_vessels
					  AS			vessels
					  ON			vessels.vessel_key = classes.class_vessel
			
					WHERE			{$vessel_filter}
					  AND			( classes.class_active_from <= :start_date OR classes.class_active_to >= :end_date )
			
					GROUP BY 		classes.class_key
						
					ORDER BY		classes.class_sort_order;
						
					";
			
			$sql = $this->prepare( $sql_query );
			
			$a_params = array( 
					":vessel"		=> $vessel,
					":start_date"	=> $start_date,
					":end_date"		=> $end_date
				);
			
			try {
				$sql->execute( $a_params );
				$a_classes = $sql->fetchAll( PDO::FETCH_OBJ );
			}
			catch ( PDOException $e ) {
				echo "Exception!";
				$a_classes = array();
				echo $e;
			}
			
			return( $a_classes );
			
		}
		
		
	
		
		
	/**
	 * Function		: getVesselBookingManager()
	 * 
	 * @param unknown $vessel
	 * 
	 * Determine the booking manager for a given vessel.
	 * 
	 */
	function getVesselBookingManager( $vessel ) {
		
		$query = "
				SELECT		vessels.ves_booking_manager
				
				FROM		`{$this->salestool_db}`.st_vessels
				  AS		vessels
				  
				WHERE		vessels.ves_name = :vessel
				";
		
		$sql = $this->prepare( $query );
		
		$a_params = array(
			":vessel"		=> $vessel
			);
		
		try {
			$sql->execute( $a_params );
		}
		catch ( PDOException $e ) {
			echo "\n\n{$e}\n\n";
			exit();
		}
	}


	/**
	 * Function		: getVesselInfo()
	 *
	 * @param $vessel_name
	 * @return bool|mixed
	 *
	 * Return some basic information about the given vessel.  Used to determine whether the vessel
	 * has multiple classes, which fleet it is in, etc, so that we can decide which query logic
	 * to use.
	 *
	 */
		function getVesselInfo( $vessel_name ) {

			$query = "
				SELECT		vessels.ves_name					AS name,
							vessels.ves_fleet					AS fleet,
							vessels.ves_booking_manager			AS booking_manager,
							vessels.ves_multi_class				AS is_multi_class,
							vessels.ves_first_published_date	AS first_published_date,
							vessels.ves_latest_published_date		AS last_published_date

				FROM		`{$this->salestool_db}`.st_vessels
					AS 		vessels

				WHERE		vessels.ves_name = :vessel_name

				LIMIT		1
				";

			$sql = $this->prepare( $query );

			$a_params = array(
				":vessel_name"		=> $vessel_name
			);

			if ( !( $sql->execute( $a_params ))) {
				$vessel_info = FALSE;
			}
			else {
				$vessel_info = $sql->fetchObject();
			}

			return( $vessel_info );
		}


	/**
	 * Function		: getVesselTrips()
	 *
	 * @param string $vessel
	 * @param string $start_date
	 * @param string $end_date
	 * @param string $classes
	 * @param string $currency
	 * @return array
	 *
	 * Get a list of trips scheduled on a vessel.
	 *
	 */
		function getVesselTrips( $vessel="", $start_date="", $end_date="", $classes="", $currency="" ) {

			// Clean up the start and end dates.
			//
			$start_date = ( $start_date == "" )
					? date( "Y-m-d")
					: date( "Y-m-d", strtotime( $start_date ));

			$end_date = ( $end_date == "" )
					? "2099-12-31"
					: date( "Y-m-d", strtotime( $end_date ));

			if ( !( $vessel_info = $this->getVesselInfo( $vessel ))) {
				$a_trips = FALSE;
			}
			else {

				switch ( $vessel_info->booking_manager ) {
					case 'BMS' :
						$a_trips = $this->getVesselTripsFromBMS(
							$vessel_info,
							$start_date,
							$end_date,
							$classes,
							$currency
						);
						break;

					case 'Paxtool' :
						$a_trips = $this->getVesselTripsFromPaxtool(
							$vessel_info,
							$start_date,
							$end_date,
							$classes,
							$currency
						);
						break;

					default :
						//  No other options to handle at this point.  Just set the result to FALSE.
						$a_trips = FALSE;
						break;
				}

			}

			return( $a_trips );
		}


	/**
	 * Function		: getVesselClassesFromBMS()
	 *
	 * @param $vessel_name
	 * @param $start_date
	 * @param $end_date
	 * @return array|bool
	 *
	 * Returns an array of class names available on the given vessel on trips departing
	 * between the start and end dates.
	 *
	 */
	function getVesselClassesFromBMS( $vessel_name, $start_date, $end_date ) {

		// TODO-Don: Get the classes to sort by lowest price available, highesty price available and then class name.  This
		// will list lowest priced cabins first.  If cabin A and B are the same price on trip 1, but for trip 2 the
		// price of cabin A is increased while cabin B remains the same, then cabin B will list before cabin A.

		$query = "
			SELECT			trip_cabin.cabin_type						AS class_name

			FROM			`{$this->bms_db}`.trip_cabin
				AS			trip_cabin

			JOIN 			`{$this->bms_db}`.trip
				AS			trip
				ON 			trip.id = trip_cabin.trip_id

			JOIN 			`{$this->bms_db}`.trip_template_profile
				AS			trip_template_profile
				ON 			trip_template_profile.trip_template_id = trip.trip_template_id

			JOIN			`{$this->bms_db}`.boat_profile
				AS			boat_profile
				ON			boat_profile.boat_id = trip.boat_id

			WHERE			boat_profile.name = :vessel_name
#				AND			trip.date_from >= :start_date
#				AND			trip.date_from <= :end_date

			GROUP BY 		class_name

			";

		$sql = $this->prepare( $query );

		$a_params = array(
			":vessel_name"		=> $vessel_name,
			":start_date"		=> $start_date,
			":end_date"			=> $end_date
		);

		if( !( $sql->execute( $a_params ))) {
			$a_classes = FALSE;
		}
		else {
			$a_classes = array();
			while( $row = $sql->FetchObject() ) {
				$a_classes[ $row->class_name ] = $row->class_name;
			}
		}

		return( $a_classes );

	}


	/**
	 * Function		: getVesselTripsFromBMS()
	 *
	 * @param $vessel_info
	 * @param $start_date
	 * @param $end_date
	 * @param $classes
	 * @param $currency
	 * @return mixed
	 *
	 * Get a list of trips, with pricing and availability for each from the BMS database.
	 *
	 */
	function getVesselTripsFromBMS(
			$vessel_info,
			$start_date,
			$end_date,
			$classes,
			$currency
	){

		//  Get a list of classes available on this vessels during the given operating dates.
		//
		$a_classes = $this->getVesselClassesFromBMS(
				$vessel_info->name,
				max( $start_date, $vessel_info->first_published_date),
				min( $end_date, $vessel_info->last_published_date )
				);

		//  Count total available spaces on a trip.
		//
		$count_trip_availability = "
			SELECT		COUNT(*)

			FROM		`{$this->bms_db}`.seat_availability
				AS		avail_seat

            JOIN        `{$this->bms_db}`.trip_cabin
              AS        avail_cabin
              ON        avail_cabin.id = avail_seat.trip_cabin_id

            WHERE		avail_seat.trip_id = trip.id
#            	AND		avail_cabin.status = 'OPEN'
            	AND		( avail_seat.status = 'OPEN' OR avail_seat.is_buy_back = '1')
			";


		$query = "
			SELECT			boat_profile.name							AS vessel_name,
							trip.code									AS trip_code,
							trip.date_from								AS trip_start_date,
							trip.date_to								AS trip_end_date,
							DATEDIFF( trip.date_to, trip.date_from )	AS trip_nights,
							trip_template_profile.name					AS trip_itinerary,
							trip_template_profile.name					AS trip_itinerary_long,
							trip.embarkation							AS trip_embarkation_point,
							trip.disembarkation							AS trip_disembarkation_point,
							0											AS price_class,
							trip_cabin.cabin_type						AS class_name,
							trip_cabin.price + trip.surcharge			AS price_amount,
							trip_cabin.currency							AS price_currency,
							( {$count_trip_availability} )				AS trip_availability

			FROM 			`{$this->bms_db}`.trip_cabin
				AS			trip_cabin

			JOIN 			`{$this->bms_db}`.trip
				AS			trip
				ON 			trip.id = trip_cabin.trip_id
			
			JOIN			`{$this->bms_db}`.trip_template
			  	AS			trip_template
				ON			trip_template.id = trip.trip_template_id

			JOIN 			`{$this->bms_db}`.trip_template_profile
				AS			trip_template_profile
				ON 			trip_template_profile.trip_template_id = trip.trip_template_id

			JOIN			`{$this->bms_db}`.boat_profile
				AS			boat_profile
				ON			boat_profile.boat_id = trip.boat_id

			JOIN			`{$this->salestool_db}`.st_vessels
				AS 			vessels
				ON			vessels.ves_name = boat_profile.name

			WHERE			boat_profile.name = :vessel_name
				AND			trip.date_from >= :start_date
				AND			trip.date_from >= vessels.ves_first_published_date
				AND			trip.date_from <= :end_date
				AND			trip.date_from <= vessels.ves_latest_published_date
				AND			trip_template.itinerary_type = 'ITINERARY'
				AND			trip.status = 'ACTIVE'

			GROUP BY 		trip_code, class_name

			ORDER BY		trip_start_date

			";

		$sql = $this->prepare( $query );

		$a_params = array(
			":vessel_name"		=> $vessel_info->name,
			":start_date"		=> $start_date,
			":end_date"			=> $end_date
		);

		if( !( $sql->execute( $a_params ))) {
			$a_trips = FALSE;
		}
		else {
			$a_trips = array();

			while( $trip = $sql->fetchObject()) {

				//  If we don't have a record for this trip code, create a new one
				//  and add it to the $a_trips array.
				//
				if ( !( isSet( $a_trips[ $trip->trip_code ] ))) {
					$trip_object = new tripClass( $trip, $a_classes );
					$a_trips[ $trip->trip_code ] = $trip_object;
				}

				//  Update the trip cabin information with the details for this
				//  class.
				//
				$a_trips[ $trip->trip_code ]->trip_prices[ $trip->class_name ]->price_amount = $trip->price_amount;

			}
		}

		return( $a_trips );
	}


	/**
	 * Function		: getVesselTripsFromPaxtool()
	 *
	 * @param $vessel_info
	 * @param $start_date
	 * @param $end_date
	 * @param $classes
	 * @param $currency
	 * @return array
	 *
	 * Get schedule and availability for vessels that are handled in Paxtool.
	 *
	 */
	function getVesselTripsFromPaxtool(
			$vessel_info,
			$start_date,
			$end_date,
			$classes,
			$currency
			){
			
				
			//  If a vessel is defined, match either the name or the key.
			//
			if ( $vessel_info == "" ) {
				$vessel = 1;
				$vessel_filter = ":vessel = :vessel";
			}
			else {
				$vessel_filter = "(( vessels.vessel_key = :vessel ) OR ( vessels.vessel_name = :vessel))";
			}
			
			$query_availability = "
					SELECT		COUNT(*)
					
					FROM		`{$this->paxtool_db}`.pax_si_belegung_details
						AS		beld
						
					JOIN		`{$this->paxtool_db}`.pax_si_belegung
						AS		bel
						ON		bel.bel_ter_key = beld.beld_ter_key
						  AND	bel.bel_platz_nr = beld.beld_platz_nr
						  
					WHERE		beld.beld_ter_key = ter.ter_key 
						AND		beld.beld_buchungsstatus = 'available'
						AND		bel.bel_kabine_status = 'public'
					";
			
			$sql_query = "
					
					SELECT			vessels.vessel_name										AS vessel_name,
									ter.ter_key												AS trip_key,
									ter.ter_code											AS trip_code,
									ter.ter_von												AS trip_start_date, 
									ter.ter_bis												AS trip_end_date, 
									DATEDIFF( ter.ter_bis, ter.ter_von)					 	AS trip_nights,
									itins.itin_name_short									AS trip_itinerary,
									itins.itin_name											AS trip_itinerary_long,
									itins.itin_embarkation_point							AS trip_embarkation_point,
									itins.itin_disembarkation_point							AS trip_disembarkation_point,
									'TBA'													AS trip_prices,
									( {$query_availability} )								AS trip_availability,
									itins.itin_key											AS trip_itin_key
									
					FROM			{$this->paxtool_db}.pax_si_termine
						AS			ter
						
					JOIN			{$this->paxtool_db}.pax_si_routen
						AS			rou
						ON			rou.rou_key = ter.ter_rou_key
						
					JOIN			{$this->paxtool_db}.pax_si_routen_details
						AS			roud
						ON			roud.roud_key = ter.ter_rou_key
						
					JOIN			{$this->schedule_db}.ml_vessels
						AS			vessels
						ON			vessels.vessel_name = ter.ter_schiff

					LEFT JOIN		{$this->salestool_db}.st_vessels
						AS 			st_vessels
						ON 			st_vessels.ves_name = ter.ter_schiff
						
					JOIN			{$this->schedule_db}.ml_itineraries
						AS			itins
						ON			itins.itin_pax_key = ter.ter_rou_key
						
					WHERE			{$vessel_filter}
						AND 		ter.ter_von >= :start_date
						AND 		ter.ter_von >= vessels.vessel_start_operation
						AND 		ter.ter_von >= st_vessels.ves_first_published_date
						AND			ter.ter_von  <= :end_date
						AND			rou.rou_typ = 'I'
			
					ORDER BY		trip_start_date;
					
					";
			
			$sql = $this->prepare( $sql_query );

			$a_params = array(
					':vessel' 		=> $vessel_info->name,
					':start_date' 	=> $start_date,
					':end_date' 	=> $end_date
					);

			try {
				$sql->execute( $a_params );
				$a_trips = $sql->fetchAll( PDO::FETCH_OBJ );
			}
			catch ( PDOException $e ) {
				echo "ERROR: SQL query failed.  " . $e->getMessage();
				$a_trips = array();
			}
			
			
// 			//  Sub query -- get the latest price start date where the price range starts before the beginning
// 			//  of the trip and ends after the beginning of the trip. 
// 			//
// 			$sub_query = "
// 				SELECT 		MAX( prices_dates.price_start_date )		AS price_start_date
				
// 				FROM		{$this->schedule_db}.ml_prices
// 					AS		prices_dates
					
// 				WHERE		prices_dates.price_start_date <= :trip_start_date
// 					AND		prices_dates.price_end_date >= :trip_start_date
// 					AND		prices_dates.price_itin = :itin_key
// 				";
			
// 			//  Now that we have the trips, for each trip, get a list of the prices for each cabin. 
// 			//

// 			$sql_query = "
			
// 				SELECT				prices.price_class					AS price_class, 
// 									classes.class_name					AS class_name,
// 									prices.price_usd					AS price_amount,
// 									'USD'								AS price_currency
					
// 				FROM 				{$this->schedule_db}.ml_prices
// 					AS				prices
					
// 				LEFT JOIN			{$this->schedule_db}.ml_vessels
// 					AS				vessels
// 					ON				vessels.vessel_key = prices.price_vessel
					
// 				LEFT JOIN			{$this->schedule_db}.ml_classes
// 					AS				classes
// 					ON				classes.class_key = prices.price_class
					
// 				LEFT JOIN 			{$this->schedule_db}.currencies
// 					AS				currency
// 					ON				currency.cur_key = prices.price_currency
					
// 				WHERE				{$vessel_filter}
// 					AND				prices.price_itin = :itin_key
// 					AND				prices.price_start_date = ( {$sub_query} )
// 					AND				prices.price_active = 'Yes'
					
// 				ORDER BY			classes.class_sort_order; 
// 				";
			
			$sql_query = "
				SELECT				prices.price_class								AS price_class, 
									classes.class_name								AS class_name,
									
									#  If there is a special price set for this trip, use it
									#  else use the normal price.  Either way add any surcharge
									#  to the trip price.
									#
									IF( special_prices.id IS NOT NULL,
										# special_prices.price_usd + surcharges.amount_usd,
										# prices.price_usd + surcharges.amount_usd
										special_prices.price_usd + IF( surcharges.id IS NULL, 0, surcharges.amount_usd ),
										prices.price_usd + IF( surcharges.id IS NULL, 0, surcharges.amount_usd )
										)											AS price_amount,
										
									'USD'											AS price_currency
					
				FROM				`{$this->paxtool_db}`.pax_si_termine
					AS				ter
					
				JOIN				`{$this->schedule_db}`.ml_itineraries
					AS				itins
					ON				itins.itin_pax_key = ter.ter_rou_key
					
				JOIN				`{$this->schedule_db}`.ml_vessels
					AS				vessels
					ON				vessels.vessel_name = ter.ter_schiff
					
				JOIN				`{$this->schedule_db}`.ml_classes
					AS				classes
					ON				classes.class_vessel = vessels.vessel_key
					
				JOIN				`{$this->schedule_db}`.ml_prices
					AS				prices
					ON				prices.price_itin = itins.itin_key
						AND			prices.price_class = classes.class_key
						
				LEFT JOIN			`{$this->schedule_db}`.ml_surcharges
					AS				surcharges
					ON				surcharges.trip_id = ter.ter_key
					
				LEFT JOIN			`{$this->schedule_db}`.ml_special_prices
					AS				special_prices
					ON				special_prices.trip_id = ter.ter_key
						AND			special_prices.class_id = classes.class_key
						
				WHERE				ter.ter_schiff = :vessel
					AND				ter.ter_key = :trip_key
					AND				prices.price_start_date <= :trip_start_date
					AND				prices.price_end_date >= :trip_start_date
					
				ORDER BY			classes.class_sort_order
					";
			
			$sql = $this->prepare( $sql_query );
			
			$sql->bindParam( ":vessel", 			$vessel_info->name );
			$sql->bindParam( ":trip_start_date", 	$trip_start_date );
			$sql->bindParam( ":trip_end_date", 		$trip_end_date );
			$sql->bindParam( ":trip_key", 			$trip_key );
						
			foreach ( $a_trips as $trip ) {

				$vessel 			= $trip->vessel_name;
				$itin_key 			= $trip->trip_itin_key;
				$trip_start_date	= $trip->trip_start_date;
				$trip_end_date		= $trip->trip_end_date;
				$trip_key			= $trip->trip_key;
				
				$sql->execute();
				
				$trip->trip_prices = $sql->fetchAll( PDO::FETCH_CLASS );
				
				unset( $trip->trip_itin_key );
			}

			return( $a_trips );
		}
		
		
		
		function getTripAvailability( $trip_code ) {
			
			//  Sub query to count the number of available spaces.
			//
			$sql_count_spaces = "

				SELECT 			COUNT(*)
				
				FROM			{$this->schedule_db}.pax_si_belegung_details
					AS			beld
					
				JOIN			{$this->schedule_db}.pax_si_belegung
					AS			bel
					ON			(( bel.bel_ter_key = beld.beld_ter_key ) AND ( bel.bel_platz_nr = beld.beld_platz_nr ))
						
				WHERE			beld.beld_ter_key 	= ter.ter_key
					AND			beld.beld_buchungsstatus = 'available'
					AND			bel.bel_kabine_status = 'public'

				";
			
			//  Main query to get the list of trips and the number of spaces 
			//  available on each.
			//
			$sql_query = "
					
					SELECT		ter.ter_code							AS trip_code,
								ter.ter_schiff							AS trip_vessel,
								roud.roud_name							AS trip_itinerary,
								ter.ter_von								AS trip_start_date,
								ter.ter_bis								AS trip_end_date,
								DATEDIFF( ter.ter_bis, ter.ter_von )	AS trip_nights,
								ter.ter_preis_1_wert					AS trip_price_eur,
								
								( {$sql_count_spaces} ) 				AS trip_spaces_available,
									
								terd.terd_bemerkung_public				AS trip_notes
					
					FROM		{$this->schedule_db}.pax_si_termine
						AS		ter
					
					JOIN		{$this->schedule_db}.pax_si_termine_details
						AS		terd
						ON		terd.terd_key = ter.ter_key
						
					JOIN		{$this->schedule_db}.pax_si_routen_details
						AS		roud
						ON		roud.roud_key = ter.ter_rou_key
						
					WHERE		ter.ter_code = :trip_code;
					";
			
			$sql_vars = array( ":trip_code" => $trip_code );
			
			$sql = $this->prepare( $sql_query );
			$sql->execute( $sql_vars );
			
			$a_results = $sql->fetchAll( PDO::FETCH_OBJ );
			
			$availability = ( count( $a_results ) == 0 ) 
				? FALSE
				: $a_results[ 0 ];
			
			return( $availability );
		}
		
	}
	?>
